Module: Cachely
- Defined in:
- lib/cachely/mechanics.rb,
lib/cachely.rb,
lib/cachely/version.rb
Overview
place all mechanics methods here bc we do not want to introduce random method namespaces to objects
Defined Under Namespace
Modules: ClassMethods, Mechanics
Constant Summary collapse
- VERSION =
"0.0.3"
Class Method Summary collapse
Class Method Details
.included(base) ⇒ Object
56 57 58 |
# File 'lib/cachely.rb', line 56 def self.included(base) base.extend ClassMethods end |